@@ -7,27 +7,30 @@ from typing import AbstractSet, Any, Callable, Dict, Mapping, Optional, Sequence
 from dagster_dbt.cli.types import DbtCliOutput
 from dagster_dbt.cli.utils import execute_cli
 from dagster_dbt.types import DbtOutput
-from dagster_dbt.utils import generate_events
+from dagster_dbt.utils import _get_input_name, _get_output_name, result_to_events
 
 from dagster import (
     AssetKey,
-    AssetMaterialization,
     AssetsDefinition,
     In,
     MetadataValue,
     Nothing,
     Out,
-    Output,
+    PartitionsDefinition,
     SolidExecutionContext,
     TableColumn,
     TableSchema,
 )
 from dagster import _check as check
 from dagster import get_dagster_logger, op
-from dagster.core.asset_defs.load_assets_from_modules import prefix_assets
-from dagster.core.definitions.events import CoercibleToAssetKeyPrefix
-from dagster.core.definitions.metadata import RawMetadataValue
-from dagster.core.errors import DagsterInvalidSubsetError
+from dagster._core.definitions.events import CoercibleToAssetKeyPrefix
+from dagster._core.definitions.load_assets_from_modules import prefix_assets
+from dagster._core.definitions.metadata import RawMetadataValue
+from dagster._core.errors import DagsterInvalidSubsetError
+from dagster._utils.backcompat import experimental_arg_warning
+
+# dbt resource types that may be considered assets
+ASSET_RESOURCE_TYPES = ["model", "seed", "snapshot"]
 
 
 def _load_manifest_for_project(
@@ -43,12 +46,13 @@ def _load_manifest_for_project(
             "project-dir": project_dir,
             "profiles-dir": profiles_dir,
             "select": select,
-            "resource-type": "model",
             "output": "json",
         },
         warn_error=False,
         ignore_handled_error=False,
         target_path=target_dir,
+        json_log_format=True,
+        capture_logs=True,
     )
     manifest_path = os.path.join(target_dir, "manifest.json")
     with open(manifest_path, "r", encoding="utf8") as f:
@@ -99,10 +103,6 @@ def _select_unique_ids_from_manifest_json(
     return selected
 
 
-def _get_node_name(node_info: Mapping[str, Any]):
-    return "__".join([node_info["resource_type"], node_info["package_name"], node_info["name"]])
-
-
 def _get_node_asset_key(node_info: Mapping[str, Any]) -> AssetKey:
     """By default:
 
@@ -134,156 +134,12 @@ def _get_node_group_name(node_info: Mapping[str, Any]) -> Optional[str]:
 def _get_node_description(node_info):
     code_block = textwrap.indent(node_info["raw_sql"], "    ")
     description_sections = [
-        node_info["description"],
+        node_info["description"] or f"dbt {node_info['resource_type']} {node_info['name']}",
         f"#### Raw SQL:\n```\n{code_block}\n```",
     ]
     return "\n\n".join(filter(None, description_sections))
 
 
-def _dbt_nodes_to_assets(
-    dbt_nodes: Mapping[str, Any],
-    select: str,
-    selected_unique_ids: AbstractSet[str],
-    runtime_metadata_fn: Optional[
-        Callable[[SolidExecutionContext, Mapping[str, Any]], Mapping[str, RawMetadataValue]]
-    ] = None,
-    io_manager_key: Optional[str] = None,
-    node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey] = _get_node_asset_key,
-    use_build_command: bool = False,
-) -> AssetsDefinition:
-
-    outs: Dict[str, Out] = {}
-    group_names: Dict[AssetKey, str] = {}
-    asset_ins: Dict[AssetKey, Tuple[str, In]] = {}
-
-    asset_deps: Dict[AssetKey, Set[AssetKey]] = {}
-
-    out_name_to_node_info: Dict[str, Mapping[str, Any]] = {}
-
-    package_name = None
-    for unique_id in selected_unique_ids:
-        cur_asset_deps = set()
-        node_info = dbt_nodes[unique_id]
-        if node_info["resource_type"] != "model":
-            continue
-        package_name = node_info.get("package_name", package_name)
-
-        for dep_name in node_info["depends_on"]["nodes"]:
-            dep_type = dbt_nodes[dep_name]["resource_type"]
-            # ignore seeds/snapshots/tests
-            if dep_type not in ["source", "model"]:
-                continue
-            dep_asset_key = node_info_to_asset_key(dbt_nodes[dep_name])
-
-            # if it's a source, it will be used as an input to this multi-asset
-            if dep_type == "source":
-                asset_ins[dep_asset_key] = (dep_name.replace(".", "_"), In(Nothing))
-
-            # regardless of type, list this as a dependency for the current asset
-            cur_asset_deps.add(dep_asset_key)
-
-        # generate the Out that corresponds to this model
-        node_name = node_info["name"]
-        outs[node_name] = Out(
-            description=_get_node_description(node_info),
-            io_manager_key=io_manager_key,
-            metadata=_get_node_metadata(node_info),
-            is_required=False,
-        )
-        out_name_to_node_info[node_name] = node_info
-
-        asset_key = node_info_to_asset_key(node_info)
-
-        # set the asset dependencies for this asset
-        asset_deps[asset_key] = cur_asset_deps
-
-        # set the group for this asset
-        group_name = _get_node_group_name(node_info)
-        if group_name is not None:
-            group_names[asset_key] = group_name
-
-    # prevent op name collisions between multiple dbt multi-assets
-    op_name = f"run_dbt_{package_name}"
-    if select != "*":
-        op_name += "_" + hashlib.md5(select.encode()).hexdigest()[-5:]
-
-    @op(
-        name=op_name,
-        tags={"kind": "dbt"},
-        ins=dict(asset_ins.values()),
-        out=outs,
-        required_resource_keys={"dbt"},
-    )
-    def dbt_op(context):
-        dbt_output = None
-
-        # clean up any run results from the last run
-        context.resources.dbt.remove_run_results_json()
-
-        # in the case that we're running everything, opt for the cleaner selection string
-        if len(context.selected_output_names) == len(outs):
-            subselect = select
-        else:
-            # for each output that we want to emit, translate to a dbt select string by converting
-            # the out to it's corresponding fqn
-            subselect = [
-                ".".join(out_name_to_node_info[out_name]["fqn"])
-                for out_name in context.selected_output_names
-            ]
-
-        try:
-            if use_build_command:
-                dbt_output = context.resources.dbt.build(select=subselect)
-            else:
-                dbt_output = context.resources.dbt.run(select=subselect)
-        finally:
-            # in the case that the project only partially runs successfully, still attempt to generate
-            # events for the parts that were successful
-            if dbt_output is None:
-                dbt_output = DbtOutput(result=context.resources.dbt.get_run_results_json())
-
-            # yield an Output for each materialization generated in the run
-            for event in generate_events(
-                dbt_output,
-                node_info_to_asset_key=node_info_to_asset_key,
-                manifest_json=context.resources.dbt.get_manifest_json(),
-            ):
-                # convert AssetMaterializations to outputs
-                if isinstance(event, AssetMaterialization):
-                    output_name = event.asset_key.path[-1]
-                    if runtime_metadata_fn:
-                        yield Output(
-                            value=None,
-                            output_name=output_name,
-                            metadata=runtime_metadata_fn(
-                                context, out_name_to_node_info[output_name]
-                            ),
-                        )
-                    else:
-                        yield Output(
-                            value=None,
-                            output_name=output_name,
-                            metadata_entries=event.metadata_entries,
-                        )
-                # yield AssetObservations normally
-                else:
-                    yield event
-
-    return AssetsDefinition(
-        keys_by_input_name={
-            input_name: asset_key for asset_key, (input_name, _) in asset_ins.items()
-        },
-        keys_by_output_name={
-            output_name: node_info_to_asset_key(out_name_to_node_info[output_name])
-            for output_name in outs.keys()
-        },
-        node_def=dbt_op,
-        can_subset=True,
-        asset_deps=asset_deps,
-        group_names_by_key=group_names,
-    )
-
-
 def _get_node_metadata(node_info: Mapping[str, Any]) -> Mapping[str, Any]:
     metadata: Dict[str, Any] = {}
     columns = node_info.get("columns", [])
@@ -303,6 +159,205 @@ def _get_node_metadata(node_info: Mapping[str, Any]) -> Mapping[str, Any]:
     return metadata
 
 
+def _get_deps(dbt_nodes, selected_unique_ids, asset_resource_types):
+
+    asset_deps: Dict[str, Set[str]] = {}
+    for unique_id in selected_unique_ids:
+        node_info = dbt_nodes[unique_id]
+        node_resource_type = node_info["resource_type"]
+
+        # skip non-asset resources, such as tests and sources
+        if node_resource_type not in asset_resource_types:
+            continue
+
+        asset_deps[unique_id] = set()
+        for parent_unique_id in node_info["depends_on"]["nodes"]:
+            parent_node_info = dbt_nodes[parent_unique_id]
+            parent_resource_type = parent_node_info["resource_type"]
+
+            # only sources or other assets may be parents
+            if parent_resource_type not in ["source"] + asset_resource_types:
+                continue
+
+            asset_deps[unique_id].add(parent_unique_id)
+
+    return asset_deps
+
+
+def _get_dbt_op(
+    op_name: str,
+    ins: Dict[str, In],
+    outs: Dict[str, Out],
+    select: str,
+    use_build_command: bool,
+    fqns_by_output_name: Dict[str, str],
+    node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey],
+    partition_key_to_vars_fn: Optional[Callable[[str], Mapping[str, Any]]],
+    runtime_metadata_fn: Optional[
+        Callable[[SolidExecutionContext, Mapping[str, Any]], Mapping[str, RawMetadataValue]]
+    ],
+):
+    @op(
+        name=op_name,
+        tags={"kind": "dbt"},
+        ins=ins,
+        out=outs,
+        required_resource_keys={"dbt"},
+    )
+    def _dbt_op(context):
+        dbt_output = None
+
+        # clean up any run results from the last run
+        context.resources.dbt.remove_run_results_json()
+
+        # in the case that we're running everything, opt for the cleaner selection string
+        if len(context.selected_output_names) == len(outs):
+            subselect = select
+        else:
+            # for each output that we want to emit, translate to a dbt select string by converting
+            # the out to its corresponding fqn
+            subselect = [
+                ".".join(fqns_by_output_name[output_name])
+                for output_name in context.selected_output_names
+            ]
+
+        try:
+            # variables to pass into the command
+            kwargs = {"select": subselect}
+            if partition_key_to_vars_fn:
+                kwargs["vars"] = partition_key_to_vars_fn(context.partition_key)
+
+            if use_build_command:
+                dbt_output = context.resources.dbt.build(**kwargs)
+            else:
+                dbt_output = context.resources.dbt.run(**kwargs)
+        finally:
+            # in the case that the project only partially runs successfully, still attempt to generate
+            # events for the parts that were successful
+            if dbt_output is None:
+                dbt_output = DbtOutput(result=context.resources.dbt.get_run_results_json())
+
+            manifest_json = context.resources.dbt.get_manifest_json()
+
+            for result in dbt_output.result["results"]:
+                if runtime_metadata_fn:
+                    node_info = manifest_json["nodes"][result["unique_id"]]
+                    extra_metadata = runtime_metadata_fn(context, node_info)
+                else:
+                    extra_metadata = None
+                yield from result_to_events(
+                    result=result,
+                    docs_url=dbt_output.docs_url,
+                    node_info_to_asset_key=node_info_to_asset_key,
+                    manifest_json=manifest_json,
+                    extra_metadata=extra_metadata,
+                    generate_asset_outputs=True,
+                )
+
+    return _dbt_op
+
+
+def _dbt_nodes_to_assets(
+    dbt_nodes: Mapping[str, Any],
+    select: str,
+    selected_unique_ids: AbstractSet[str],
+    runtime_metadata_fn: Optional[
+        Callable[[SolidExecutionContext, Mapping[str, Any]], Mapping[str, RawMetadataValue]]
+    ] = None,
+    io_manager_key: Optional[str] = None,
+    node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey] = _get_node_asset_key,
+    use_build_command: bool = False,
+    partitions_def: Optional[PartitionsDefinition] = None,
+    partition_key_to_vars_fn: Optional[Callable[[str], Mapping[str, Any]]] = None,
+    node_info_to_group_fn: Callable[[Dict[str, Any]], Optional[str]] = _get_node_group_name,
+) -> AssetsDefinition:
+
+    asset_deps: Dict[AssetKey, Set[AssetKey]] = {}
+
+    asset_ins: Dict[AssetKey, Tuple[str, In]] = {}
+    asset_outs: Dict[AssetKey, Tuple[str, Out]] = {}
+
+    group_names_by_key: Dict[AssetKey, str] = {}
+    fqns_by_output_name: Dict[str, str] = {}
+
+    package_name: str = ""
+
+    if use_build_command:
+        deps = _get_deps(
+            dbt_nodes, selected_unique_ids, asset_resource_types=["model", "seed", "snapshot"]
+        )
+    else:
+        deps = _get_deps(dbt_nodes, selected_unique_ids, asset_resource_types=["model"])
+
+    for unique_id, parent_unique_ids in deps.items():
+        node_info = dbt_nodes[unique_id]
+
+        output_name = _get_output_name(node_info)
+        fqns_by_output_name[output_name] = node_info["fqn"]
+
+        asset_key = node_info_to_asset_key(node_info)
+
+        asset_deps[asset_key] = set()
+
+        asset_outs[asset_key] = (
+            output_name,
+            Out(
+                io_manager_key=io_manager_key,
+                description=_get_node_description(node_info),
+                metadata=_get_node_metadata(node_info),
+                is_required=False,
+            ),
+        )
+
+        package_name = node_info.get("package_name", package_name)
+
+        group_name = node_info_to_group_fn(node_info)
+        if group_name is not None:
+            group_names_by_key[asset_key] = group_name
+
+        for parent_unique_id in parent_unique_ids:
+            parent_node_info = dbt_nodes[parent_unique_id]
+            parent_asset_key = node_info_to_asset_key(parent_node_info)
+
+            asset_deps[asset_key].add(parent_asset_key)
+
+            # if this parent is not one of the selected nodes, it's an input
+            if parent_unique_id not in deps:
+                input_name = _get_input_name(parent_node_info)
+                asset_ins[parent_asset_key] = (input_name, In(Nothing))
+
+    # prevent op name collisions between multiple dbt multi-assets
+    op_name = f"run_dbt_{package_name}"
+    if select != "*":
+        op_name += "_" + hashlib.md5(select.encode()).hexdigest()[-5:]
+
+    dbt_op = _get_dbt_op(
+        op_name=op_name,
+        ins=dict(asset_ins.values()),
+        outs=dict(asset_outs.values()),
+        select=select,
+        use_build_command=use_build_command,
+        fqns_by_output_name=fqns_by_output_name,
+        node_info_to_asset_key=node_info_to_asset_key,
+        partition_key_to_vars_fn=partition_key_to_vars_fn,
+        runtime_metadata_fn=runtime_metadata_fn,
+    )
+
+    return AssetsDefinition(
+        keys_by_input_name={
+            input_name: asset_key for asset_key, (input_name, _) in asset_ins.items()
+        },
+        keys_by_output_name={
+            output_name: asset_key for asset_key, (output_name, _) in asset_outs.items()
+        },
+        node_def=dbt_op,
+        can_subset=True,
+        asset_deps=asset_deps,
+        group_names_by_key=group_names_by_key,
+        partitions_def=partitions_def,
+    )
+
+
 def load_assets_from_dbt_project(
     project_dir: str,
     profiles_dir: Optional[str] = None,
@@ -316,20 +371,23 @@ def load_assets_from_dbt_project(
     io_manager_key: Optional[str] = None,
     node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey] = _get_node_asset_key,
     use_build_command: bool = False,
+    partitions_def: Optional[PartitionsDefinition] = None,
+    partition_key_to_vars_fn: Optional[Callable[[str], Mapping[str, Any]]] = None,
+    node_info_to_group_fn: Callable[[Dict[str, Any]], Optional[str]] = _get_node_group_name,
 ) -> Sequence[AssetsDefinition]:
     """
-    Loads a set of DBT models from a DBT project into Dagster assets.
+    Loads a set of dbt models from a dbt project into Dagster assets.
 
     Creates one Dagster asset for each dbt model. All assets will be re-materialized using a single
-    `dbt run` command.
+    `dbt run` or `dbt build` command.
 
     Args:
-        project_dir (Optional[str]): The directory containing the DBT project to load.
+        project_dir (Optional[str]): The directory containing the dbt project to load.
         profiles_dir (Optional[str]): The profiles directory to use for loading the DBT project.
             Defaults to a directory called "config" inside the project_dir.
-        target_dir (Optional[str]): The target directory where DBT will place compiled artifacts.
+        target_dir (Optional[str]): The target directory where dbt will place compiled artifacts.
             Defaults to "target" underneath the project_dir.
-        select (Optional[str]): A DBT selection string for the models in a project that you want
+        select (Optional[str]): A dbt selection string for the models in a project that you want
             to include. Defaults to "*".
         key_prefix (Optional[Union[str, List[str]]]): A prefix to apply to all models in the dbt
             project. Does not apply to sources.
@@ -342,10 +400,18 @@ def load_assets_from_dbt_project(
             assets. When other ops are downstream of the loaded assets, the IOManager specified
             here determines how the inputs to those ops are loaded. Defaults to "io_manager".
         node_info_to_asset_key: (Mapping[str, Any] -> AssetKey): A function that takes a dictionary
-            of dbt node info and returns the AssetKey that you want to represent that node. By
-            default, the asset key will simply be the name of the dbt model.
-        use_build_command: (bool): Flag indicating if you want to use `dbt build` as the core computation
+            of dbt metadata and returns the AssetKey that you want to represent a given model or
+            source. By default: dbt model -> AssetKey([model_name]) and
+            dbt source -> AssetKey([source_name, table_name])
+        use_build_command (bool): Flag indicating if you want to use `dbt build` as the core computation
             for this asset, rather than `dbt run`.
+        partitions_def (Optional[PartitionsDefinition]): Defines the set of partition keys that
+            compose the dbt assets.
+        partition_key_to_vars_fn (Optional[str -> Dict[str, Any]]): A function to translate a given
+            partition key (e.g. '2022-01-01') to a dictionary of vars to be passed into the dbt
+            invocation (e.g. {"run_date": "2022-01-01"})
+        node_info_to_group_fn (Dict[str, Any] -> Optional[str]): A function that takes a
+            dictionary of dbt node info and returns the group that this node should be assigned to.
 
     """
     project_dir = check.str_param(project_dir, "project_dir")
@@ -363,19 +429,23 @@ def load_assets_from_dbt_project(
     )
     return load_assets_from_dbt_manifest(
         manifest_json=manifest_json,
+        select=select,
         key_prefix=key_prefix,
         source_key_prefix=source_key_prefix,
         runtime_metadata_fn=runtime_metadata_fn,
         io_manager_key=io_manager_key,
         selected_unique_ids=selected_unique_ids,
-        select=select,
         node_info_to_asset_key=node_info_to_asset_key,
         use_build_command=use_build_command,
+        partitions_def=partitions_def,
+        partition_key_to_vars_fn=partition_key_to_vars_fn,
+        node_info_to_group_fn=node_info_to_group_fn,
     )
 
 
 def load_assets_from_dbt_manifest(
     manifest_json: Mapping[str, Any],
+    select: Optional[str] = None,
     key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
     source_key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
     runtime_metadata_fn: Optional[
@@ -383,9 +453,11 @@ def load_assets_from_dbt_manifest(
     ] = None,
     io_manager_key: Optional[str] = None,
     selected_unique_ids: Optional[AbstractSet[str]] = None,
-    select: Optional[str] = None,
     node_info_to_asset_key: Callable[[Mapping[str, Any]], AssetKey] = _get_node_asset_key,
     use_build_command: bool = False,
+    partitions_def: Optional[PartitionsDefinition] = None,
+    partition_key_to_vars_fn: Optional[Callable[[str], Mapping[str, Any]]] = None,
+    node_info_to_group_fn: Callable[[Dict[str, Any]], Optional[str]] = _get_node_group_name,
 ) -> Sequence[AssetsDefinition]:
     """
     Loads a set of dbt models, described in a manifest.json, into Dagster assets.
@@ -396,6 +468,8 @@ def load_assets_from_dbt_manifest(
     Args:
         manifest_json (Optional[Mapping[str, Any]]): The contents of a DBT manifest.json, which contains
             a set of models to load into assets.
+        select (Optional[str]): A dbt selection string for the models in a project that you want
+            to include. Defaults to "*".
         key_prefix (Optional[Union[str, List[str]]]): A prefix to apply to all models in the dbt
             project. Does not apply to sources.
         source_key_prefix (Optional[Union[str, List[str]]]): A prefix to apply to all sources in the
@@ -411,10 +485,26 @@ def load_assets_from_dbt_manifest(
         node_info_to_asset_key: (Mapping[str, Any] -> AssetKey): A function that takes a dictionary
             of dbt node info and returns the AssetKey that you want to represent that node. By
             default, the asset key will simply be the name of the dbt model.
-        use_build_command: (bool): Flag indicating if you want to use `dbt build` as the core computation
+        use_build_command (bool): Flag indicating if you want to use `dbt build` as the core computation
             for this asset, rather than `dbt run`.
+        partitions_def (Optional[PartitionsDefinition]): Defines the set of partition keys that
+            compose the dbt assets.
+        partition_key_to_vars_fn (Optional[str -> Dict[str, Any]]): A function to translate a given
+            partition key (e.g. '2022-01-01') to a dictionary of vars to be passed into the dbt
+            invocation (e.g. {"run_date": "2022-01-01"})
+        node_info_to_group_fn (Dict[str, Any] -> Optional[str]): A function that takes a
+            dictionary of dbt node info and returns the group that this node should be assigned to.
     """
     check.dict_param(manifest_json, "manifest_json", key_type=str)
+    if partitions_def:
+        experimental_arg_warning("partitions_def", "load_assets_from_dbt_manifest")
+    if partition_key_to_vars_fn:
+        experimental_arg_warning("partition_key_to_vars_fn", "load_assets_from_dbt_manifest")
+        check.invariant(
+            partitions_def is not None,
+            "Cannot supply a `partition_key_to_vars_fn` without a `partitions_def`.",
+        )
+
     dbt_nodes = {**manifest_json["nodes"], **manifest_json["sources"]}
 
     if select is None:
@@ -438,6 +528,9 @@ def load_assets_from_dbt_manifest(
         selected_unique_ids=selected_unique_ids,
         node_info_to_asset_key=node_info_to_asset_key,
         use_build_command=use_build_command,
+        partitions_def=partitions_def,
+        partition_key_to_vars_fn=partition_key_to_vars_fn,
+        node_info_to_group_fn=node_info_to_group_fn,
     )
     if source_key_prefix:
         if isinstance(source_key_prefix, str):
