@@ -1,10 +1,10 @@
 import json
 import os
 import subprocess
-from typing import Any, Dict, Optional
+from typing import Any, Dict, Optional, Sequence
 
 import dagster._check as check
-from dagster.core.utils import coerce_valid_log_level
+from dagster._core.utils import coerce_valid_log_level
 
 from ..errors import (
     DagsterDbtCliFatalRuntimeError,
@@ -15,6 +15,40 @@ from .constants import DBT_RUN_RESULTS_COMMANDS, DEFAULT_DBT_TARGET_PATH
 from .types import DbtCliOutput
 
 
+def _create_command_list(
+    executable: str,
+    warn_error: bool,
+    json_log_format: bool,
+    command: str,
+    flags_dict: Dict[str, Any],
+) -> Sequence[str]:
+
+    prefix = [executable]
+    if warn_error:
+        prefix += ["--warn-error"]
+    if json_log_format:
+        prefix += ["--no-use-color", "--log-format", "json"]
+
+    full_command = command.split(" ")
+    for flag, value in flags_dict.items():
+        if not value:
+            continue
+
+        full_command.append(f"--{flag}")
+
+        if isinstance(value, bool):
+            pass
+        elif isinstance(value, list):
+            check.list_param(value, f"config.{flag}", of_type=str)
+            full_command += value
+        elif isinstance(value, dict):
+            full_command.append(json.dumps(value))
+        else:
+            full_command.append(str(value))
+
+    return prefix + full_command
+
+
 def execute_cli(
     executable: str,
     command: str,
@@ -24,81 +58,92 @@ def execute_cli(
     ignore_handled_error: bool,
     target_path: str,
     docs_url: Optional[str] = None,
+    json_log_format: bool = True,
+    capture_logs: bool = True,
 ) -> DbtCliOutput:
     """Executes a command on the dbt CLI in a subprocess."""
+    try:
+        import dbt  # pylint: disable=unused-import
+    except ImportError as e:
+        raise check.CheckError(
+            "You must have `dbt-core` installed in order to execute dbt CLI commands."
+        ) from e
+
     check.str_param(executable, "executable")
     check.str_param(command, "command")
     check.dict_param(flags_dict, "flags_dict", key_type=str)
     check.bool_param(warn_error, "warn_error")
     check.bool_param(ignore_handled_error, "ignore_handled_error")
+    check.bool_param(capture_logs, "capture_logs")
 
-    # Format the dbt CLI flags in the command..
-    warn_error = ["--warn-error"] if warn_error else []
-    command_list = [
-        executable,
-        "--no-use-color",
-        "--log-format",
-        "json",
-        *warn_error,
-        *command.split(" "),
-    ]
-
-    for flag, value in flags_dict.items():
-        if not value:
-            continue
-
-        command_list.append(f"--{flag}")
-
-        if isinstance(value, bool):
-            # If a bool flag (and is True), the presence of the flag itself is enough.
-            continue
-
-        if isinstance(value, list):
-            check.list_param(value, f"config.{flag}", of_type=str)
-            command_list += value
-            continue
-
-        if isinstance(value, dict):
-            command_list.append(json.dumps(value))
-            continue
-
-        command_list.append(str(value))
+    command_list = _create_command_list(
+        executable=executable,
+        warn_error=warn_error,
+        json_log_format=json_log_format,
+        command=command,
+        flags_dict=flags_dict,
+    )
 
     # Execute the dbt CLI command in a subprocess.
     full_command = " ".join(command_list)
-    log.info(f"Executing command: {full_command}")
+    log.info(f"Executing command: {' '.join(command_list)}")
 
     return_code = 0
-    logs = []
-    output = []
 
-    process = subprocess.Popen(command_list, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    # raw lines
+    raw_output = []
+
+    # parsed messages from the log output
+    messages = []
+
+    # json dictionaries for each line (if applicable)
+    json_lines = []
+
+    process = subprocess.Popen(
+        command_list,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.STDOUT,
+    )
     for raw_line in process.stdout or []:
-        line = raw_line.decode("utf-8")
-        output.append(line)
-        try:
-            json_line = json.loads(line)
-        except json.JSONDecodeError:
-            log.info(line.rstrip())
-        else:
-            logs.append(json_line)
-            level = coerce_valid_log_level(
-                json_line.get("levelname", json_line.get("level", "info"))
-            )
-            log.log(level, json_line.get("message", json_line.get("msg", line.rstrip())))
+        line = raw_line.decode("utf-8").rstrip()
+
+        log_level = "info"
+        message = line
+
+        raw_output.append(line)
+
+        if json_log_format:
+            try:
+                json_line = json.loads(line)
+                json_lines.append(json_line)
+            except json.JSONDecodeError:
+                pass
+            else:
+                # in rare cases, the loaded json line may be a string rather than a dictionary
+                if isinstance(message, dict):
+                    message = json_line.get("message", json_line.get("msg", message))
+                    log_level = json_line.get("levelname", json_line.get("level", "debug"))
+
+        messages.append(message)
+        if capture_logs:
+            log.log(coerce_valid_log_level(log_level), message)
 
     process.wait()
     return_code = process.returncode
 
     log.info("dbt exited with return code {return_code}".format(return_code=return_code))
 
-    raw_output = "\n".join(output)
+    raw_output = "\n\n".join(raw_output)
 
     if return_code == 2:
-        raise DagsterDbtCliFatalRuntimeError(logs=logs, raw_output=raw_output)
+        raise DagsterDbtCliFatalRuntimeError(
+            logs=json_lines, raw_output=raw_output, messages=messages
+        )
 
     if return_code == 1 and not ignore_handled_error:
-        raise DagsterDbtCliHandledRuntimeError(logs=logs, raw_output=raw_output)
+        raise DagsterDbtCliHandledRuntimeError(
+            logs=json_lines, raw_output=raw_output, messages=messages
+        )
 
     run_results = (
         parse_run_results(flags_dict["project-dir"], target_path)
@@ -110,7 +155,7 @@ def execute_cli(
         command=full_command,
         return_code=return_code,
         raw_output=raw_output,
-        logs=logs,
+        logs=json_lines,
         result=run_results,
         docs_url=docs_url,
     )
