@@ -22,8 +22,8 @@ from dagster import (
     String,
 )
 from dagster import _check as check
-from dagster import solid
-from dagster.core.execution.context.compute import SolidExecutionContext
+from dagster._core.execution.context.compute import SolidExecutionContext
+from dagster._legacy import solid
 
 from ..errors import DagsterDbtRpcUnexpectedPollOutputError
 from .types import DbtRpcOutput
